﻿using System;
using System.Reflection;

namespace showfiles {
  class Program {
    static void Main(string[] args) {
      int argCount = args.Length;
      if (argCount == 1 ||  argCount == 2) {
        string arg1 = args[0];
        if (arg1[0] == '-') {
          switch (arg1[1]) {
            case 't':
              if (argCount == 2) {
                System.Diagnostics.Stopwatch sw = 
                  new System.Diagnostics.Stopwatch();
                sw.Start();

                ShowFiles(args[1]);
                Console.Error.WriteLine(
                  "showfile command finished (" +
                  sw.ElapsedMilliseconds.ToString() 
                  + " msec)");
              } else {
                ShowUsage();
              }
              break;
            case 'v':
              ShowVersion();
              break;
            default:
              ShowUsage();
              break;
          }
        } else {
            ShowFiles(arg1);
        }
      } else {
        ShowUsage();
      }
    }

    static void ShowUsage() {
      Console.WriteLine(
        "usage: showfiles [options] target_directory" + 
        Environment.NewLine +
        "option(s): -v display version information" + 
        Environment.NewLine +
        "           -h output elapsed time to error");
    }

    static void ShowVersion() {
      Assembly assembly = 
        Assembly.GetExecutingAssembly();
      string version = 
        assembly.GetName().Version.ToString();
      string exePath = assembly.Location.ToString();
      System.Diagnostics.FileVersionInfo fvi = (
        System.Diagnostics.FileVersionInfo.
        GetVersionInfo(exePath));
      string copyright = fvi.LegalCopyright.ToString();

      Console.WriteLine(
        "showfiles by C# Version "
        + version + "(2011/09/06)" + 
        Environment.NewLine +
        copyright + " All rights reserved");
    }

    static void ShowFiles(string target) {
      if (System.IO.Directory.Exists(target)) {
        string[] files = { };
        string[] directories = { };
        try {
          directories = System.IO.Directory.
            GetDirectories(target);
          foreach (string directory in directories)
            ShowFiles(directory);
          files = 
            System.IO.Directory.GetFiles(target);
        } catch (Exception e) {
          Console.WriteLine(
            "error: " + e.Message.ToString());
        }

        foreach (string file in files)
          Console.WriteLine(file);
      } else {
        Console.WriteLine("error : \"" + target +
          "\" is not a valid directory");
      }
    }
  }
}
