﻿using System;
using System.Reflection;

namespace which {
  class Program {
    static void Main(string[] args) {
      if (args.Length == 1) {
        string arg1 = args[0];
        if (arg1[0] == '-') {
          if (arg1[1] == 'v') {
            ShowVersion();
          } else {
            ShowUsage();
          }
        } else {
          Console.WriteLine(FindExecutable(arg1));
        }
      } else {
        ShowUsage();
      }
    }

    static void ShowUsage() {
      Console.WriteLine(
        "usage: which [options] commandname" + 
        Environment.NewLine +
        "option(s): -v display version information");
    }

    static void ShowVersion() {
      Assembly assembly = 
        Assembly.GetExecutingAssembly();
      string version = 
        assembly.GetName().Version.ToString();
      string exePath = assembly.Location.ToString();
      System.Diagnostics.FileVersionInfo fvi = (
        System.Diagnostics.FileVersionInfo.
        GetVersionInfo(exePath));
      string copyright = 
        fvi.LegalCopyright.ToString();

      Console.WriteLine(
        "which by C# Version " + version + 
        "(2011/09/06)" + Environment.NewLine +
        copyright + " All rights reserved");
    }

    static string FindExecutable(string keyword) {
      if (-1 != keyword.IndexOf('.')) 
        return "error: the keyword includes dot(.)";
      string path = 
        Environment.GetEnvironmentVariable("path");
      path = System.Environment.CurrentDirectory + 
        ';' + path;
      string[] paths = path.Split(';');
      string[] executableExtensions = 
        { ".COM", ".EXE", ".BAT", ".CMD" };
      string[] possibleFilenames = 
        new string[executableExtensions.Length];
      string ucaseKeyword = keyword.ToUpper();
      for (int i = 0; 
        i < executableExtensions.Length; i++)
        possibleFilenames[i] = 
          ucaseKeyword + executableExtensions[i];

      foreach (string s in paths) {
        string[] files = 
          System.IO.Directory.GetFiles(s);
        foreach (string t in files) {
          string filename = 
            System.IO.Path.GetFileName(t).ToUpper();
          foreach (string pfn in possibleFilenames)
            if (pfn == filename) return t;
        }
      }
      return "cannot find \"" + keyword + "\"";
    }
  }
}
